#!/usr/bin/python

import gnumeric
import commands
import sys
import tempfile
import getpass
import string
import smtplib
import time

from wxPython.wx import *
from wxPython.grid import *

# If we had to convert a non .gnumeric spreadsheet, then we want to get
# rid of the temp before quitting.  Leaving copies of gradebooks laying
# around is not cool.
deleteArg1 = [False]

SELECT_USERNAMES = 1
SELECT_DATA_RANGE = 2
SELECT_HEADERS = 3
SELECT_FIRSTNAMES = 4
SELECT_POSSIBLE = 5
SEND = 6

#---------------------------------------------------------------------------

class PreambleDialog(wxDialog):
    '''This class lets the user enter the preamble message in the grade
    mailback'''
    def __init__(self, parent):
        # Inherit from wxDialog
        wxDialog.__init__(self, parent, -1, "Preamble", size=wxSize(320, 200))

        # Store who our parent is
        self.parent = parent

        # We have a blank panel
        p = wxPanel(self, -1, style=0)

        # A text box with a default message
        tb = wxTextCtrl(p, -1,
                        "Here is a summary of your current scores.",
                        size=wxSize(300, 160),
                        style=wxTE_MULTILINE|wxTE_WORDWRAP)
        self.tb = tb

        # An OK button
        b = wxButton(p, -1, "OK")

        # Place the widgets
        bs = wxBoxSizer(wxVERTICAL)
        bs.Add(tb, 1, wxGROW|wxALL, 5)
        bs.Add(b)
        p.SetSizer(bs)

        # Set up event handling
        EVT_BUTTON(self, b.GetId(), self.OnOK)

    def OnOK(self, event):
        # Store the preamble
        self.parent.preamble = self.tb.GetValue()
        self.EndModal(wxID_OK)

class CustomDataTable(wxPyGridTableBase):
    """Stolen shamelessly from the wxPython demo files. """

    def __init__(self, log, data):
        wxPyGridTableBase.__init__(self)
        self.log = log

        lens = []
        for row in data:
            lens.append(len(row))

        # We are only dealing with strings here
        self.dataTypes = [wxGRID_VALUE_STRING] * max(lens)

        for ind in range(len(data)):
            data[ind] = map(str, data[ind])

        # If the CSV conversion added '"' chars to the ends of the
        # string, then strip those off, they are ugly.
        for j in range(len(data)):
            for ind, i in enumerate(data[j]):
                if i and '"' == i[0] and '"' == i[-1]:
                    data[j][ind] = data[j][ind][1:-1]

        # Set the column labels
        self.colLabels = data[0][:]
        data.pop(0)

        # Note our data
        self.data = data[:]

        


    #--------------------------------------------------
    # required methods for the wxPyGridTableBase interface


    def GetNumberRows(self):
        return len(self.data) + 1

    def GetNumberCols(self):
        return len(self.data[0])

    def IsEmptyCell(self, row, col):
        try:
            return not self.data[row][col]
        except IndexError:
            return true

    # Get/Set values in the table.  The Python version of these
    # methods can handle any data-type, (as long as the Editor and
    # Renderer understands the type too,) not just strings as in the
    # C++ version.  However, we are only doing get, since we don't
    # want to edit this stuff
    def GetValue(self, row, col):
        try:
            return self.data[row][col]
        except IndexError:
            return ''

    def SetValue(self, row, col, value):
        pass


    #--------------------------------------------------
    # Some optional methods

    # Called when the grid needs to display labels
    def GetColLabelValue(self, col):
        return self.colLabels[col]

    # Called to determine the kind of editor/renderer to use by
    # default, doesn't necessarily have to be the same type used
    # nativly by the editor/renderer if they know how to convert.
    def GetTypeName(self, row, col):
        return self.dataTypes[col]

    # Called to determine how the data can be fetched and stored by the
    # editor and renderer.  This allows you to enforce some type-safety
    # in the grid.
    def CanGetValueAs(self, row, col, typeName):
        colType = self.dataTypes[col].split(':')[0]
        if typeName == colType:
            return true
        else:
            return False

    def CanSetValueAs(self, row, col, typeName):
        return self.CanGetValueAs(row, col, typeName)


#---------------------------------------------------------------------------


class CustTableGrid(wxGrid):
    '''A grid widget with a CustDataTable underneath it'''
    def __init__(self, panel, log, filename, sheet, parent):
        wxGrid.__init__(self, panel, -1)

        # Do the conversion to get values from the spreadsheet
        # (this way we deref all formulas)
        fn = tempfile.mkstemp(suffix=".csv")[1]
        gnumeric.selectSheet(filename, sheet)
        
        commands.getstatusoutput("ssconvert %s %s" % (filename, fn))

        # Read the data
        data = []
        for line in open(fn):
            data.append(line.split(","))

        commands.getstatusoutput("rm " + fn)

        table = CustomDataTable(log, data)
        self.table = table

        # The second parameter means that the grid is to take ownership of the
        # table and will destroy it when done.
        # Otherwise you would need to keep
        # a reference to it and call it's Destroy method later.
        self.SetTable(table, true)

        self.SetRowLabelSize(0)
        self.SetMargins(0,0)
        self.AutoSizeColumns(False)

        self.parent = parent
        
        # Track selection changes
        EVT_GRID_RANGE_SELECT(self, self.onSelectRange)
        EVT_GRID_SELECT_CELL(self, self.onSelect)


    def onSelectRange(self, event):
        x1, y1 = event.GetLeftCol(), event.GetTopRow()
        x2, y2 = event.GetRightCol(), event.GetBottomRow()

        self.parent.upper = y1
        self.parent.lower = y2
        self.parent.right = x2
        self.parent.left = x1

    def onSelect(self, event):
        x1 = x2 = event.GetCol()
        y1 = y2 = event.GetRow()

        self.parent.upper = y1
        self.parent.lower = y2
        self.parent.right = x2
        self.parent.left = x1

        event.Allow()
        event.Skip()
        
#---------------------------------------------------------------------------

class TestFrame(wxFrame):
    def __init__(self, parent, log):
        wxFrame.__init__(self, parent, -1,
                         "Spreadsheet Mailback", size=(800,600))
        p = wxPanel(self, -1, style=0)

        # If we don't have any arguments, find the file we want to work on
        if len(sys.argv) == 1:
            dlg = wxFileDialog(self, "Select spreadsheet gradebook",
                               style=wxOPEN)
            result = dlg.ShowModal()
            file = dlg.GetFilename()
            dlg.Destroy()
            if result != wxID_OK:
                self.Close()
                return
            sys.argv.append(file)

        # If our file is not a .gnumeric file, then convert to gnumeric
        if sys.argv[1].split(".")[-1] != "gnumeric":
            fn = tempfile.mkstemp(suffix=".gnumeric")[1]
            commands.getstatusoutput("rm " + fn)
            (status, output) = commands.getstatusoutput("ssconvert %s %s" % \
                                                        (sys.argv[1], fn))
            if status:
                dlg = wxMessageDialog(self, output, "Unable to convert!",
                                      style=wxOK|wxICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
                sys.exit(0)
            sys.argv[1] = fn
            global deleteArg1
            # Note that we need to get rid of our cleanup
            deleteArg1[0] = True

        # Choose the sheet
        dlg = wxSingleChoiceDialog(self, "Mailback from which sheet?",
                                   "Select Sheet",
                                   gnumeric.sheetNames(sys.argv[1]))
        if dlg.ShowModal() == wxID_CANCEL:
            dlg.Destroy()
            sys.exit(0)

        # Now we have the sheet index
        sheet = dlg.GetSelection()
        dlg.Destroy()

        # Build the widgets
        grid = CustTableGrid(p, log, sys.argv[1], sheet, self)
        self.grid = grid

        # Turn off editing on the grid
        grid.EnableEditing(False)
        grid.EnableDragGridSize(False)

        # The starting state of our mighty button
        b = wxButton(p, -1, "Set username range")
        b.SetDefault()
        self.button = b        
        EVT_BUTTON(self, b.GetId(), self.OnButton)

        # Pack it in
        bs = wxBoxSizer(wxVERTICAL)
        bs.Add(grid, 1, wxGROW|wxALL, 5)
        bs.Add(b)
        p.SetSizer(bs)

        # Not our current state
        self.mode = SELECT_USERNAMES
        self.upper = None
        self.lower = None
        self.right = None
        self.left = None

        self.firstNames = None
        self.maxVals = None

    def OnButton(self, evt):
        '''The workhorse of the app.  Update everything that needs updating
        when clicked, then call send when we are done setting values/ranges.'''

        if self.mode == SELECT_USERNAMES:
            if None in (self.upper, self.lower, self.right, self.left):
                dlg = wxMessageDialog(self,
                                      "Must select a range containing the " +\
                                       "usernames",
                                      "Error",
                                      style=wxICON_ERROR|wxOK)
                dlg.ShowModal()
                dlg.Destroy()

                return

            if self.left != self.right:
                dlg = wxMessageDialog(self,
                                      "Must select a single column",
                                      "Error",
                                      style=wxICON_ERROR|wxOK)
                dlg.ShowModal()
                dlg.Destroy()
                return
              

            self.usernameRange = self.left, self.upper, self.lower
            self.button.SetLabel("Set data range")
            self.mode = SELECT_DATA_RANGE

            for i in range(self.usernameRange[1], self.usernameRange[2] + 1):
                self.grid.SetCellBackgroundColour(i, self.usernameRange[0],
                                                  wxColour(128, 0, 0))
                self.grid.SetCellTextColour(i, self.usernameRange[0],
                                            wxColour(255, 255, 255))
                
            self.grid.ForceRefresh()
            self.grid.SelectBlock(self.upper, self.left, self.upper, self.left)
            
        elif self.mode == SELECT_DATA_RANGE:
            if self.upper < self.usernameRange[1] or \
               self.lower > self.usernameRange[2]:
                dlg = wxMessageDialog(self,
                                      "Must select data range within rows " +\
                                      "given by username selection.",
                                      "Error",
                                      style=wxICON_ERROR|wxOK)
                dlg.ShowModal()
                dlg.Destroy()
                return

            self.dataRange = self.left, self.right
            self.button.SetLabel("Set header row")
            self.mode = SELECT_HEADERS

            for i in range(self.left, self.right + 1):
                for j in range(self.usernameRange[1],
                               self.usernameRange[2] + 1):
                    self.grid.SetCellBackgroundColour(j, i,
                                                      wxColour(0, 0, 128))
                    self.grid.SetCellTextColour(j, i,
                                                wxColour(255, 255, 255))

            self.grid.ForceRefresh()

            dlg = wxMessageDialog(self,
                                  "Use column headers? (Choose 'No' to " + \
                                  "select alternate row)",
                                  "Question",
                                  style=wxYES_NO|wxICON_QUESTION)

            response = dlg.ShowModal()
            dlg.Destroy()
            self.grid.SelectBlock(self.upper, self.left, self.upper, self.left)
        
            if response == wxID_YES:
                self.left = self.dataRange[0]
                self.right = self.dataRange[1]
                self.upper = self.lower = -1
            else:
                return
                        
        if self.mode == SELECT_HEADERS:
            
            if self.upper != self.lower or \
               self.usernameRange[1] < self.upper < self.usernameRange[2]:
                dlg = wxMessageDialog(self,
                                      "Must select unique header row",
                                      "Error",
                                      style=wxICON_ERROR|wxOK)
                dlg.ShowModal()
                dlg.Destroy()

            self.headerRange = self.upper

            for col in range(self.dataRange[0], self.dataRange[1] + 1):
                self.grid.SetCellBackgroundColour(self.headerRange, col,
                                                  wxColour(0, 128, 0))
                self.grid.SetCellTextColour(self.headerRange, col,
                                            wxColour(255, 255, 255))

            self.grid.SelectBlock(self.upper, self.left, self.upper, self.left)
        
            self.grid.ForceRefresh()

            # Firstnames
            dlg = wxMessageDialog(self,
                                  "Select firstnames col",
                                  "Question",
                                  style=wxYES_NO|wxICON_QUESTION)
            response = dlg.ShowModal()
            dlg.Destroy()
            if response == wxID_YES:
                self.button.SetLabel("Set firstname col")
                self.mode = SELECT_FIRSTNAMES
                return
            
            # Max values
            dlg = wxMessageDialog(self,
                                  "Select max values row?",
                                  "Question",
                                  style=wxYES_NO|wxICON_QUESTION)
            response = dlg.ShowModal()
            dlg.Destroy()
            if response == wxID_YES:
                self.button.SetLabel("Set max value row")
                self.mode = SELECT_POSSIBLE
                return            

            self.mode = SEND

        if self.mode == SELECT_FIRSTNAMES:
            if self.left != self.right or \
               self.dataRange[0] < self.right < self.dataRange[1] or \
               self.left == self.usernameRange[0]:
                dlg = wxMessageDialog(self,
                                      "Must select unused column.",
                                      "Error",
                                      style=wxOK|wxICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
                return
            
            self.firstNames = self.left

            for row in range(self.usernameRange[1], self.usernameRange[2] + 1):
                self.grid.SetCellBackgroundColour(row, self.firstNames, 
                                                  wxColour(128, 128, 0))
                self.grid.SetCellTextColour(row, self.firstNames, 
                                            wxColour(255, 255, 255))

            self.grid.SelectBlock(self.upper, self.left, self.upper, self.left)
            self.grid.ForceRefresh()

            #  Max values?
            dlg = wxMessageDialog(self,
                                  "Select max values row?",
                                  "Question",
                                  style=wxYES_NO|wxICON_QUESTION)
            response = dlg.ShowModal()
            dlg.Destroy()
            if response == wxID_YES:
                self.button.SetLabel("Set max value row")
                self.mode = SELECT_POSSIBLE
                return            

            self.mode = SEND
            
        if self.mode == SELECT_POSSIBLE:
            if self.upper != self.lower or \
               self.usernameRange[1] < self.upper < self.usernameRange[2] or \
               self.upper == self.headerRange:
                dlg = wxMessageDialog(self,
                                      "Must select unused row.",
                                      "Error",
                                      style=wxOK|wxICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
                return
            
            self.maxVals = self.upper

            for col in range(self.dataRange[0], self.dataRange[1] + 1):
                self.grid.SetCellBackgroundColour(self.maxVals, col,
                                                  wxColour(128, 0, 128))
                self.grid.SetCellTextColour(self.maxVals, col,
                                            wxColour(255, 255, 255))

            self.grid.SelectBlock(self.upper, self.left, self.upper, self.left)
            self.grid.ForceRefresh()
            
            self.mode = SEND

        if self.mode == SEND:
            dlg = PreambleDialog(self)
            dlg.ShowModal()
            dlg.Destroy()
            self.send()

    def send(self):
        usernameRange = self.usernameRange
        headerRange = self.headerRange
        dataRange = self.dataRange
        
        first = True
        cur = 0
        progress = None
        for i in range(usernameRange[1], usernameRange[2] + 1):
            message = "From: %s@cs.ucr.edu\n" % getpass.getuser()
            
            
            username = self.grid.table.GetValue(i, self.usernameRange[0])
            username = string.strip(username)
            if not username:
                cur += 1
                continue
            
            if "@" not in username:
                username += "@cs.ucr.edu"

            if progress:
                cur += 1
                progress.Update(cur, username)

            message += "To: %s\n" % username
            message += "Subject: Current Scores\n"
            
            if self.firstNames != None:
                name = self.grid.table.GetValue(i, self.firstNames)
                message += name + ",\n"
            
            message += self.preamble + "\n"
            
            for col in range(dataRange[0], dataRange[1] + 1):
                if headerRange == -1:
                    message += self.grid.table.GetColLabelValue(col)
                else:
                    message += self.grid.table.GetValue(self.headerRange, col)

                message += ": " + self.grid.table.GetValue(i, col)

                if self.maxVals != None:
                    message +=  "/ " + \
                               self.grid.table.GetValue(self.maxVals, col)
                message += "\n"

            if first:
                dlg = wxMessageDialog(self,
                                      "This is what the mailbacks " + \
                                      "will look like.\n\n" + message + \
                                      "\n\nSend now?",
                                      "Approve",
                                      style=wxOK|wxCANCEL|wxICON_QUESTION)
                response = dlg.ShowModal()
                dlg.Destroy()
                if response != wxID_OK:
                    self.Close()
                    return

                num = usernameRange[2] + 1 - usernameRange[1]
                progress = wxProgressDialog("Sending", username, num)

                first = False
                server = smtplib.SMTP("mail.cs.ucr.edu")

                
            server.sendmail(getpass.getuser() + "@cs.ucr.edu",
                            username,
                            message)
            time.sleep(.25)

                                            
        progress.Close()
        progress.Destroy()

        dlg = wxMessageDialog(self,
                              "Messages sent.  Check your mailbox for\n" +\
                              "possible bounces.",
                              "Done.",
                              style=wxOK|wxICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

        self.Close()
            

#---------------------------------------------------------------------------

if __name__ == '__main__':
    import sys
    
    app = wxPySimpleApp()
    frame = TestFrame(None, sys.stdout)
    frame.Show(true)
    app.MainLoop()

    if deleteArg1[0]:
        commands.getoutput("rm " + sys.argv[1])

#---------------------------------------------------------------------------
